package org.embulk.parser.header_based_csv.output

import org.embulk.parser.header_based_csv.HeaderBasedCsvParserPlugin
import org.embulk.spi.`type`._
import org.embulk.spi.json.JsonParser
import org.embulk.spi.time.TimestampParser
import org.embulk.spi.util.Timestamps
import org.embulk.spi.{Column, PageBuilder}
import helper.GoogleOptionalHelper.RichOptional

import scala.util.{Failure, Try}

class ColumnValueWriter(task: HeaderBasedCsvParserPlugin#PluginTask, pageBuilder: PageBuilder) {

  private lazy val timestampParsers = Timestamps.newTimestampColumnParsers(task, task.getSchemaConfig)
  private lazy val jsonParser       = new JsonParser()

  def writeValue(column: Column, value: String): Try[Unit] = column.getType match {
    case _: LongType      => writeLong(column, value)
    case _: DoubleType    => writeDouble(column, value)
    case _: StringType    => writeString(column, value)
    case _: TimestampType => writeTimestamp(column, value)(timestampParsers)
    case _: JsonType      => writeJson(column, value)
    case _                => Failure(new UnsupportedOperationException(""))
  }

  def writeNull(column: Column): Try[Unit] = Try(pageBuilder.setNull(column))

  private[output] def writeString(column: Column, value: String): Try[Unit] = {
    task.getNullString.toOption match {
      case Some(strToBeNull) if value.matches(strToBeNull) =>
        Try(pageBuilder.setNull(column))
      case _ =>
        Try(pageBuilder.setString(column, value))
    }
  }

  private[output] def writeBoolean(column: Column, value: String): Try[Unit] =
    Try(pageBuilder.setBoolean(column, value.toBoolean))

  private[output] def writeLong(column: Column, value: String): Try[Unit] =
    Try(pageBuilder.setLong(column, value.toLong))

  private[output] def writeDouble(column: Column, value: String): Try[Unit] =
    Try(pageBuilder.setDouble(column, value.toDouble))

  private[output] def writeTimestamp(column: Column,
                                     value: String)(timestampParsers: Array[TimestampParser]): Try[Unit] =
    Try {
      val parser = timestampParsers(column.getIndex)
      pageBuilder.setTimestamp(column, parser.parse(value))
    }

  private[output] def writeJson(column: Column, value: String): Try[Unit] =
    Try(pageBuilder.setJson(column, jsonParser.parse(value)))

}
